package com.link.core.basic.es.util;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.link.core.basic.es.annotation.EsField;
import com.link.core.basic.model.BasicModel;
import com.link.core.basic.service.BasicServiceException;
import com.link.core.cllog.LCLogger;
import com.link.core.util.SpringUtil;
import com.link.core.util.StringUtils;
import java.lang.reflect.Field;
import java.sql.Date;
import java.sql.Time;
import java.sql.Timestamp;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Optional;
import java.util.stream.Collectors;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.util.CollectionUtils;

public class EsUtil {
   private static final Logger logger = LogManager.getLogger(EsUtil.class);
   public static final String CHILD = "child";
   public static final String PARENT = "parent";

   public static Map<String, String> parseNestedField(String fieldName, BasicModel entity) {
      Map<String, String> resultMap = new HashMap();
      resultMap.put(EsFieldType.NESTED_OBJECT.getValue(), fieldName);
      if (fieldName.contains(".")) {
         return resultMap;
      } else {
         Class<?> className = entity.getClass();
         Field declaredField = null;

         while(className != Object.class) {
            try {
               declaredField = className.getDeclaredField(fieldName);
               break;
            } catch (NoSuchFieldException var7) {
               if (className == BasicModel.class) {
                  return resultMap;
               }

               className = className.getSuperclass();
            }
         }

         EsField annotation = (EsField)declaredField.getAnnotation(EsField.class);
         if (null != annotation && EsFieldType.NESTED_OBJECT.equals(annotation.type())) {
            String result = annotation.value();
            resultMap.put(EsFieldType.NESTED_OBJECT.getValue(), result);
            if (!result.contains(".")) {
               logger.error("当前注解@EsField.value格式设置有误，字段名：{}, 注解值：{}", fieldName, result);
               throw new BasicServiceException("查询字段解析格式有误");
            }
         } else if (null != annotation && EsFieldType.RUNTIME.equals(annotation.type())) {
            resultMap.put(EsFieldType.RUNTIME.getValue(), annotation.runtimeType().getValue());
         }

         return resultMap;
      }
   }

   public static void initAggrParams(BasicModel basicModel, EsAggregateType aggregateType, String groupFields, String aggregationField) {
      if (!StringUtils.isBlank(groupFields) && !StringUtils.isBlank(aggregationField) && null != aggregateType) {
         try {
            basicModel.setAggregationType(aggregateType.getValue());
            basicModel.setGroupFields(groupFields);
            basicModel.setAggregationField(aggregationField);
         } catch (Exception var5) {
            Exception e = var5;
            logger.error("QueryParamUtil: ", e);
            throw new BasicServiceException("初始化聚合参数失败：" + e.getMessage());
         }
      } else {
         throw new BasicServiceException("初始化聚合查询参数缺失");
      }
   }

   public static void initCountParams(BasicModel basicModel, String groupFields) {
      try {
         initAggrParams(basicModel, EsAggregateType.COUNT, groupFields, "id");
      } catch (Exception var3) {
         Exception e = var3;
         logger.error("QueryParamUtil: ", e);
         throw new BasicServiceException("初始化聚合参数失败：" + e.getMessage());
      }
   }

   public static List<Map<String, Object>> queryDataById(BasicModel entity, String exIndex) {
      JdbcTemplate jdbcTemplate = (JdbcTemplate)SpringUtil.getBean(JdbcTemplate.class);

      try {
         Map<String, Object> esConfigMap = EsCacheUtils.getEsConfigCache(exIndex);
         String sql;
         Object childConfig;
         if ("child".equals(entity.getEsDocShipFlag())) {
            if (StringUtils.isBlank(entity.getEsDocShipMark())) {
               throw new BasicServiceException("更新子文档信息，需要给定子文档标识");
            }

            childConfig = esConfigMap.get("childConfig");
            if (Objects.isNull(childConfig)) {
               throw new BasicServiceException("更新子文档信息，需要给定子文档标识");
            }

            JSONArray objects = JSONArray.parseArray(JSONArray.toJSONString(childConfig));
            List<Object> childMarks = (List)objects.stream().filter((s) -> {
               return entity.getEsDocShipMark().equals(JSONObject.parseObject(JSON.toJSONString(s)).getString("childMark"));
            }).collect(Collectors.toList());
            if (CollectionUtils.isEmpty(childMarks)) {
               throw new BasicServiceException("找不到子文档【" + entity.getEsDocShipMark() + "】的配置信息");
            }

            sql = JSONObject.parseObject(JSON.toJSONString(childMarks.get(0))).getString("sql");
            if (StringUtils.isBlank(sql)) {
               throw new BasicServiceException("还没有配置【" + exIndex + "】索引的子文档【" + entity.getEsDocShipMark() + "】配置信息，请您联系管理员配置");
            }
         } else {
            childConfig = esConfigMap.get("sql");
            if (Objects.isNull(childConfig) || StringUtils.isBlank(childConfig.toString())) {
               throw new BasicServiceException("还没有配置【" + exIndex + "】索引的配置信息，请您联系管理员配置");
            }

            sql = childConfig.toString();
         }

         List<Map<String, Object>> sourcesJsonArr = jdbcTemplate.queryForList(sql, new Object[]{entity.getId()});
         if (CollectionUtils.isEmpty(sourcesJsonArr)) {
            throw new BasicServiceException("当前【" + entity.getId() + "】为查找到数据");
         } else {
            sourcesJsonArr.forEach((map) -> {
               Iterator var1 = map.entrySet().iterator();

               while(var1.hasNext()) {
                  Map.Entry<String, Object> entry = (Map.Entry)var1.next();
                  String key = (String)entry.getKey();
                  Object value = entry.getValue();
                  DateFormat df_timestamp = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                  DateFormat df_date = new SimpleDateFormat("yyyy-MM-dd");
                  DateFormat df_time = new SimpleDateFormat("HH:mm:ss");
                  String format = value instanceof Timestamp ? df_timestamp.format(value) : (value instanceof Date ? df_date.format(value) : (value instanceof Time ? df_time.format(value) : null));
                  if (StringUtils.isNotBlank(format)) {
                     map.put(key, format);
                  }

                  if (key.startsWith("arr_")) {
                     String tempValue = Optional.ofNullable(entry.getValue()).orElse("").toString();
                     List<Object> objects = tempValue.length() <= 2 ? new ArrayList() : new ArrayList(Arrays.asList(tempValue.substring(1, tempValue.length() - 1).split(",")));
                     map.put(key.substring(key.indexOf("_") + 1), objects);
                     map.remove(key);
                  }
               }

            });
            return sourcesJsonArr;
         }
      } catch (Exception var8) {
         Exception e = var8;
         LCLogger.withException(e);
         throw new BasicServiceException("根据【" + entity.getId() + "】为查找到数据异常:" + e.getMessage());
      }
   }

   public static List<Map<String, Object>> querySubNestDataById(List<Map<String, Object>> data, String exIndex) throws Exception {
      JdbcTemplate jdbcTemplate = (JdbcTemplate)SpringUtil.getBean(JdbcTemplate.class);
      List<Map<String, Object>> subConfigCaches = EsCacheUtils.getEsSubConfigCache(exIndex);
      Iterator var4 = subConfigCaches.iterator();

      while(true) {
         Map map;
         String esForeignKey;
         String tagField;
         List subList;
         do {
            String subSql;
            List foreignValues;
            do {
               if (!var4.hasNext()) {
                  return data;
               }

               map = (Map)var4.next();
               esForeignKey = map.get("esForeignKey").toString();
               tagField = map.get("tagField").toString();
               subSql = map.get("subSql").toString();
               foreignValues = (List)data.stream().filter((s) -> {
                  return s.containsKey(esForeignKey) && Objects.nonNull(s.get(esForeignKey));
               }).map((s) -> {
                  return s.get(esForeignKey);
               }).collect(Collectors.toList());
            } while(CollectionUtils.isEmpty(foreignValues));

            String condition = foreignValues.get(0) instanceof String ? (String)foreignValues.stream().map(Object::toString).collect(Collectors.joining("','")) : (String)foreignValues.stream().map(Object::toString).collect(Collectors.joining(","));
            subList = jdbcTemplate.queryForList(subSql, new Object[]{condition});
         } while(CollectionUtils.isEmpty(subList));

         subList.forEach((s) -> {
            Iterator var1 = s.entrySet().iterator();

            while(var1.hasNext()) {
               Map.Entry<String, Object> entry = (Map.Entry)var1.next();
               String key = (String)entry.getKey();
               Object value = entry.getValue();
               DateFormat df_timestamp = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
               DateFormat df_date = new SimpleDateFormat("yyyy-MM-dd");
               DateFormat df_time = new SimpleDateFormat("HH:mm:ss");
               String format = value instanceof Timestamp ? df_timestamp.format(value) : (value instanceof Date ? df_date.format(value) : (value instanceof Time ? df_time.format(value) : null));
               if (StringUtils.isNotBlank(format)) {
                  s.put(key, format);
               }
            }

         });
         List<String> foreignFieldNames = JSONArray.parseArray(JSONArray.toJSONString(map.get("foreignFieldName")), String.class);
         Iterator var13 = foreignFieldNames.iterator();

         label55:
         while(var13.hasNext()) {
            String foreignFieldName = (String)var13.next();
            Map<Object, List<Map<String, Object>>> collectMap = (Map)subList.stream().collect(Collectors.groupingBy((itemx) -> {
               return itemx.get(foreignFieldName);
            }));
            Iterator var16 = data.iterator();

            while(true) {
               while(true) {
                  if (!var16.hasNext()) {
                     continue label55;
                  }

                  Map<String, Object> item = (Map)var16.next();
                  String foreignValue = item.get(esForeignKey).toString();
                  if (item.containsKey(esForeignKey) && item.get(esForeignKey) != null) {
                     List<Map<String, Object>> maps = (List)collectMap.get(foreignValue);
                     if (CollectionUtils.isEmpty(maps)) {
                        item.put(tagField, new ArrayList());
                     } else {
                        item.put(tagField, maps);
                     }
                  } else {
                     item.put(tagField, new ArrayList());
                  }
               }
            }
         }
      }
   }

   public static String getParentFieldValue(Map<String, Object> item, String parentField) {
      List<String> parentFields = (List)Arrays.stream(parentField.split(",")).map(String::toLowerCase).collect(Collectors.toList());
      Iterator var3 = item.keySet().iterator();

      String fieldName;
      do {
         if (!var3.hasNext()) {
            throw new BasicServiceException("子文档信息中不包含父文档id信息");
         }

         fieldName = (String)var3.next();
      } while(!parentFields.contains(fieldName.toLowerCase()));

      return item.get(fieldName).toString();
   }
}
